import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/objectWithoutPropertiesLoose";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import _extends from "@babel/runtime/helpers/extends";
import * as React from 'react';
import { FixedSizeList } from 'react-window';
import { Row } from './utils';

var computeTree = function computeTree(_temp, _ref, _ref2) {
  var _ref3 = _temp === void 0 ? {} : _temp,
      _ref3$refreshNodes = _ref3.refreshNodes,
      refreshNodes = _ref3$refreshNodes === void 0 ? false : _ref3$refreshNodes,
      _ref3$useDefaultOpenn = _ref3.useDefaultOpenness,
      useDefaultOpenness = _ref3$useDefaultOpenn === void 0 ? false : _ref3$useDefaultOpenn;

  var treeWalker = _ref.treeWalker;
  var prevRecords = _ref2.records,
      recomputeTree = _ref2.recomputeTree;
  var order = [];

  var records = _extends({}, prevRecords);

  var iter = treeWalker(refreshNodes);

  if (useDefaultOpenness) {
    for (var id in records) {
      records[id].isOpen = records[id].data.isOpenByDefault;
    }
  }

  var isPreviousOpened = false;

  while (true) {
    var _iter$next = iter.next(isPreviousOpened),
        done = _iter$next.done,
        value = _iter$next.value;

    if (done || !value) {
      break;
    }

    var _id = void 0;

    if (typeof value === 'string' || typeof value === 'symbol') {
      _id = value;

      if (useDefaultOpenness) {
        records[_id].isOpen = records[_id].data.isOpenByDefault;
      }
    } else {
      _id = value.id;
      var isOpenByDefault = value.isOpenByDefault;
      var record = records[_id];

      if (!record) {
        records[_id] = {
          data: value,
          isOpen: isOpenByDefault,
          toggle: function toggle() {
            try {
              var _this2 = this;

              _this2.isOpen = !_this2.isOpen;
              return Promise.resolve(recomputeTree({
                refreshNodes: _this2.isOpen
              })).then(function () {});
            } catch (e) {
              return Promise.reject(e);
            }
          }
        };
      } else {
        record.data = value;

        if (useDefaultOpenness) {
          record.isOpen = isOpenByDefault;
        }
      }
    }

    order.push(_id);
    isPreviousOpened = records[_id].isOpen;
  }

  return {
    order: order,
    records: records
  };
};

var FixedSizeTree =
/*#__PURE__*/
function (_React$PureComponent) {
  _inheritsLoose(FixedSizeTree, _React$PureComponent);

  FixedSizeTree.getDerivedStateFromProps = function getDerivedStateFromProps(props, state) {
    var component = props.children,
        treeData = props.itemData,
        treeWalker = props.treeWalker;
    var oldTreeWalker = state.treeWalker,
        order = state.order;
    return _extends({
      component: component,
      treeData: treeData
    }, treeWalker !== oldTreeWalker || !order ? computeTree({
      refreshNodes: true
    }, props, state) : null);
  };

  function FixedSizeTree(props, context) {
    var _this3;

    _this3 = _React$PureComponent.call(this, props, context) || this;
    _this3.list = React.createRef();
    _this3.state = {
      component: props.children,
      recomputeTree: _this3.recomputeTree.bind(_assertThisInitialized(_this3)),
      records: {},
      treeWalker: props.treeWalker
    };
    return _this3;
  }

  var _proto = FixedSizeTree.prototype;

  _proto.recomputeTree = function recomputeTree(options) {
    try {
      var _this5 = this;

      return Promise.resolve(new Promise(function (resolve) {
        _this5.setState(function (prevState) {
          return computeTree(options, _this5.props, prevState);
        }, resolve);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.scrollTo = function scrollTo(scrollOffset) {
    var _this$list$current;

    (_this$list$current = this.list.current) == null ? void 0 : _this$list$current.scrollTo(scrollOffset);
  };

  _proto.scrollToItem = function scrollToItem(id, align) {
    var _this$list$current2;

    (_this$list$current2 = this.list.current) == null ? void 0 : _this$list$current2.scrollToItem(this.state.order.indexOf(id) || 0, align);
  };

  _proto.render = function render() {
    var _this$props = this.props,
        children = _this$props.children,
        treeWalker = _this$props.treeWalker,
        rowComponent = _this$props.rowComponent,
        rest = _objectWithoutPropertiesLoose(_this$props, ["children", "treeWalker", "rowComponent"]);

    return React.createElement(FixedSizeList, Object.assign({}, rest, {
      itemData: this.state,
      itemCount: this.state.order.length,
      ref: this.list
    }), rowComponent);
  };

  return FixedSizeTree;
}(React.PureComponent);

FixedSizeTree.defaultProps = {
  rowComponent: Row
};
export { FixedSizeTree as default };